## About ##

This repository contains implementation and source codes of the following paper:

[A new EM algorithm for flexibly tied GMMs with large number of components](https://www.sciencedirect.com/science/article/abs/pii/S0031320321000236)

## Acknowledgement ##

The proposed GMM model and corresponding EM algorithms are implemented using the following library:

[MixEst: A MATLAB toolbox for mixture-model parameter estimation](https://github.com/utvisionlab/mixest)



## Running The Codes ##

* Unzip and copy the whole code directory you just downloaded in a location of your choice on disk, say, in /my/directory/.

* Go to /my/directory/ at the Matlab command prompt and execute 'install_mixest'. You may save this path for your next Matlab sessions: follow the menu File . Set Path... and save.

* Go to /my/directory/thirdparty/lightspeed/ at the Matlab command prompt and execute 'install_lightspeed'.



## Directory structure ##

<pre>
The top directory with README.md
 | matlab/           - folder containing the Matlab implemetations
 | pytorch/          - folder containing the Pytorch implemetations

The matlab directory
 | tests/           - test functions 
 | doc/             - toolbox documentation
 | experiments/     - folder containing main part of experiments 
   |--- dataset/                        - folder containing datasets
   |--- component_experiments.m         - file containing the codes to compare effect of the number of components on the fitting power of models.
   |--- likelihood_experiments.m        - file containing the codes to compare the fitting power of models for different datasets.
   |--- time_experiments.m        	  	- file containing the codes to compare the computation time of different algorithms.
   |--- clustering_experiments.m        	  	- file containing the codes to compare the clustering efficiency of different algorithms.
   |--- load_dataset.m        		 	    - file containing the code to load datasets.
 | mixest/          - folder containing main toolbox files
   |--- auxiliary/        - functions needed for creating a distribution or estimating its parameters
   |--- distributions/    - a collection of functions for creating distribution structures
   |--- estimation/       - main function for estimating the parameters
   |--- gates/            - gate factories needed for creating mixtures of experts
 | thirdParty/      - third party tools
   |--- lightspeed/       - logsumexp function of T. Minka's matlab toolbox
   |--- randraw/          - function for sampling from several distributions by A. Bar-Guy
   |--- vmfmatlab/        - toolbox by A. Banerjee and S. Sra used for sampling from von Mises-Fisher distribution
   |--- manopt/           - manifold optimization toolbox by N. Boumal and B. Mishra
   |--- matlab-xunit/     - xUnit tests for Matlab
   
</pre>

## HOW TO CITE ##

<pre>
@article{asheri2021new,
  title={A new EM algorithm for flexibly tied GMMs with large number of components},
  author={Asheri, Hadi and Hosseini, Reshad and Araabi, Babak Nadjar},
  journal={Pattern Recognition},
  volume={114},
  pages={107836},
  year={2021},
  publisher={Elsevier}
}
</pre>


As this research was conducted using t please the MixEst library, please citte the following paper as well:
<pre>
@article{hosseini2015mixest,
  title={Mixest: An estimation toolbox for mixture models},
  author={Hosseini, Reshad and Mash'al, Mohamadreza},
  journal={arXiv preprint arXiv:1507.06065},
  year={2015}
}
</pre>

-------------------------------------------------------------------------------
