function component_experiments()
	close all
	clear
	clc
	datasets = {'magic_10', 'wave_21', 'letter_16'};
	out_dir = 'results';
	for idx_dataset=1:1:3
		% repeat experiment for dataset with different random data shufflings
		for exp_itr=1:1:1
			dataset = datasets(idx_dataset);
			evaluate_likelihoods(dataset, exp_itr, out_dir);
		end
	end
end

function evaluate_likelihoods(dataset, exp_itr, out_dir)
 %% load data
 disp(strcat('dataset: ', dataset))
 data = load_dataset(dataset);
 data = double(data);
 dim = size(data, 1);
 dim2 = dim;
 ndata = size(data, 2);
 disp(size(data))

 %% normalize data
 data = data(:, randperm(size(data, 2)));
 mu = mean(data, 2);
 data = bsxfun(@minus, data, mu);
 cv = cov(data');
 data = sqrtm(cv)\data;
 ndata = size(data, 2); 
 %% measuring time complexity
 for num_coef=5:5:10
		num = num_coef
		disp(strcat('num: ', num2str(num)))
		%continue
		theta_stc_initial=[];
		D_STC = rmixturefactory(rmvnfactory(dim, dim2), num);
		D_Newton = rmixturefactory_shared(rmvnfactory_shared(dim, dim2), num);
		options = {};
		options.rho = 0.1; options.lsc = 1e-4; options.lsc2 = 9e-1;
		options.cost_threshold = 1e-8;
		options.grad_threshold = 1e-8;
		options.max_ndata = 10000000;
		options.use_prior = 1;
		options.type_prior = 1;
		options.use_merit = 1;
		options.use_shared = 1;
		options.totalItrNo = 300;
		mkdir_if_not_exist(out_dir)


		d = dim;
		K = num_coef;
		P = K - 1 + K*d + d*d + K*d;

		options.totalItrNo = 300;
		num = round((P + 1) / (1 + d + d*(d + 1) / 2));
		log_dir = strcat(out_dir, '/component/exp-', num2str(exp_itr, '%03d'), '__', dataset{1}, '__gmm__coef-', num2str(num_coef, '%02d'), '__dim-', num2str(dim, '%03d'), '__num-' , num2str(num, '%03d'));			
		D = mixturefactory(mvnfactory(dim), num);
		[theta] = gmm_estimation(D, data, num, options, dataset, log_dir);

		num = num_coef
		options.pivot_itr = 0;
		log_dir = strcat(out_dir, '/component/exp-', num2str(exp_itr, '%03d'), '__', dataset{1}, '__nwt__coef-', num2str(num_coef, '%02d'), '__dim-', num2str(dim, '%03d'), '__num-' , num2str(num, '%03d'));
		[theta_Newton, theta0, theta_pivot] = estimatelinesearchsharedMAPMixedS(data, D_Newton, options, D_STC, theta_stc_initial, dataset, log_dir);

		options.pivot_itr = 0.9* options.totalItrNo;
		log_dir = strcat(out_dir, '/component/exp-', num2str(exp_itr, '%03d'), '__', dataset{1}, '__snw__coef-', num2str(num_coef, '%02d'), '__dim-', num2str(dim, '%03d'), '__num-' , num2str(num, '%03d'));
		[theta_Newton, theta0, theta_pivot] = estimatelinesearchsharedMAPMixedS(data, D_Newton, options, D_STC, theta_stc_initial, dataset, log_dir);

		theta_stc_initial = theta_pivot;
		options.totalItrNo = options.totalItrNo - options.pivot_itr;
		options.pivot_itr = 1+options.totalItrNo;
		log_dir = strcat(out_dir, '/component/exp-', num2str(exp_itr, '%03d'), '__', dataset{1}, '__stc__coef-', num2str(num_coef, '%02d'), '__dim-', num2str(dim, '%03d'), '__num-' , num2str(num, '%03d'));
		[theta_Newton, theta0, theta_pivot] = estimatelinesearchsharedMAPMixedS(data, D_Newton, options, D_STC, theta_stc_initial, dataset, log_dir);

	 
 end
end
%exit
