function clustering_experiments()
	close all
	clear
	clc
	datasets = {'dim6', 'dim9', 'dim12', 'dim15'};
	out_dir = 'results/';
	for idx_dataset=1:1:4
		for exp_itr=1:1:3
			dataset = datasets(idx_dataset);
			evaluate_clustering(dataset, exp_itr, out_dir);
		end
	end
end
%_-
function evaluate_clustering(dataset, exp_itr, out_dir)
	data_path = strcat('./dataset/', dataset{1}, 	'.txt')
	scorefile = strcat(out_dir, 'scores.txt');
	disp(data_path)
	X = importdata(data_path, ' ');
	disp(size(X))
	data = X';
	N = size(X, 1);
	ndata = N;
	dim = size(X, 2);
	dim2 = dim;

	% d= dim;
	% num_coef = 1;
	% num = num_coef * dim;
	% k = num;
	% p = k - 1 + k*d + d*d + k*d;
	% num2 = round((p + 1) / (1 + d + d*(d + 1) / 2));
			

	num_coef = 1;
	d = dim/3;
	k = d;
	num2 = k;
	p = k - 1 + k*d*(d+1)/2 + k*d;
	num = (p - (d*d-1))/(1+2*d);
	num = round(num);



	disp(strcat('num:', num2str(num)))
	disp(strcat('num2:', num2str(num2)))
	% return
	D_STC = rmixturefactory(rmvnfactory(dim, dim2), num);
	D_Newton = rmixturefactory_shared(rmvnfactory_shared(dim, dim2), num);

	itr_eval = 1;
	itr_run = 1;
	options = {};
	options.rho = 0.1; options.lsc = 1e-4; options.lsc2 = 9e-1;
	options.cost_threshold = 1e-8;
	options.grad_threshold = 1e-8;
	options.max_ndata = 100000000;
	options.use_prior = 1 ;
	options.use_merit = 1;
	options.use_shared = 1;
	options.type_prior = 1;
	idx_prior = options.type_prior

	options0 = statset('MaxIter', 300);

	gmm_sml = fitgmdist(X, num2, 'CovarianceType', 'full', 'SharedCovariance', false, 'RegularizationValue', 0.001, 'MaxIter', 300);%, 'Options', options); % Fitted GMM
	gmm_big = fitgmdist(X, num, 'CovarianceType', 'full', 'SharedCovariance', false, 'RegularizationValue', 0.001, 'MaxIter', 300);%, 'Options', options); % Fitted GMM

	theta_stc_initial=[];
	options.totalItrNo = 300;
	options.pivot_itr = 0.9 * options.totalItrNo;

	log_dir = strcat(out_dir, '/clustering/exp-', num2str(exp_itr, '%03d'), '_', dataset{1}, '_snw_priortype-', num2str(idx_prior), '_coef-', num2str(num_coef, '%02d'), '_dim-', num2str(dim, '%03d'), '_num-' , num2str(num, '%03d'));
	mkdir_if_not_exist(log_dir)

	[theta_Newton, theta0, theta_pivot, theta_set_newton_stc] = estimatelinesearchsharedMAPMixedS(data, D_Newton, options, D_STC, theta_stc_initial, dataset, log_dir);
	theta_file = strcat(log_dir, '/', dataset, '_theta.mat');
	theta_file = string(theta_file);
	save(theta_file, 'theta_set_newton_stc');

	Sigma = zeros(dim, dim, num);
	Mu = zeros(num, dim);
	p = zeros(num, 1);
	k2 = 0;
	for k=1:num
		if(theta_Newton.p(k)==0)
			continue;
		end
		k2 = k2 + 1;
		p(k2) = theta_Newton.p(k);
		Sigma(:, :, k2) = theta_Newton.U * diag(theta_Newton.D{k}.d) * theta_Newton.U';
		Mu(k2, :) = theta_Newton.D{k}.mu';
	end
	Sigma = Sigma(:, :, 1:k2);
	Mu = Mu(1:k2, :);
	p = p(1:k2);
	gmm_snw = gmdistribution(Mu, Sigma, p);

	clust_snw = cluster(gmm_snw, X); % Cluster index
	clust_gmm_sml = cluster(gmm_sml, X); % Cluster index 
	clust_gmm_big = cluster(gmm_big, X); % Cluster index 
	kmeans_sml = kmeans(X, num2);
	kmeans_big = kmeans(X, num);

	criterion = 'silhouette';
	eval = evalclusters(X, clust_snw, criterion);
	silhouette_snw = eval.CriterionValues;
	eval = evalclusters(X, clust_gmm_sml, criterion);
	silhouette_gmm_sml = eval.CriterionValues;
	eval = evalclusters(X, clust_gmm_big, criterion);
	silhouette_gmm_big = eval.CriterionValues;
	eval = evalclusters(X, kmeans_sml, criterion);
	silhouette_kmeans_sml = eval.CriterionValues;
	eval = evalclusters(X, kmeans_big, criterion);
	silhouette_kmeans_big = eval.CriterionValues;
	% disp("--------------------------------")
	disp("silhouette")
	disp(silhouette_snw)
	disp(silhouette_gmm_big)
	disp(silhouette_gmm_sml)
	disp(silhouette_kmeans_sml)
	disp(silhouette_kmeans_big)
	
	text = strcat(dataset{1}, ', ', num2str(num), ', ', num2str(num2), ', silhouette, ', num2str(silhouette_snw), ', ', num2str(silhouette_gmm_sml), ', ', num2str(silhouette_gmm_big), ', ', num2str(silhouette_kmeans_sml), ', ', num2str(silhouette_kmeans_big));
	fid = fopen(scorefile, 'a+');
 fprintf(fid, '%s\n', text);
 fclose(fid);

	

	% disp("--------------------------------")
	criterion = 'DaviesBouldin';
	eval = evalclusters(X, clust_snw, criterion);
	daviesbouldin_snw = eval.CriterionValues;
	eval = evalclusters(X, clust_gmm_sml, criterion);
	daviesbouldin_gmm_sml = eval.CriterionValues;
	eval = evalclusters(X, clust_gmm_big, criterion);
	daviesbouldin_gmm_big = eval.CriterionValues;
	eval = evalclusters(X, kmeans_sml, criterion);
	daviesbouldin_kmeans_sml = eval.CriterionValues;
	eval = evalclusters(X, kmeans_big, criterion);
	daviesbouldin_kmeans_big = eval.CriterionValues;

	text = strcat(dataset{1}, ', ', num2str(num), ', ', num2str(num2), ', daviesbouldin, ', num2str(daviesbouldin_snw), ', ', num2str(daviesbouldin_gmm_sml), ', ', num2str(daviesbouldin_gmm_big), ', ', num2str(daviesbouldin_kmeans_sml), ', ', num2str(daviesbouldin_kmeans_big));
	fid = fopen(scorefile, 'a+');
 fprintf(fid, '%s\n', text);
 fclose(fid);



	% disp("--------------------------------")
	distM=squareform(pdist(X));
	dunns_snw = dunns(num, distM, clust_snw);
	dunns_gmm_sml = dunns(num2, distM, clust_gmm_sml);
	dunns_gmm_big = dunns(num, distM, clust_gmm_big);
	dunns_kmeans_sml = dunns(num2, distM, kmeans_sml);
	dunns_kmeans_big = dunns(num, distM, kmeans_big);

	text = strcat(dataset{1}, ', ', num2str(num), ', ', num2str(num2), ', dunns, ', num2str(dunns_snw), ', ', num2str(dunns_gmm_sml), ', ', num2str(dunns_gmm_big), ', ', num2str(dunns_kmeans_sml), ', ', num2str(dunns_kmeans_big));
	fid = fopen(scorefile, 'a+');
 fprintf(fid, '%s\n', text);
 fclose(fid);

	disp(strcat('num:', num2str(num)))
	disp(strcat('num2:', num2str(num2)))

end

function gmm = generate_gmm(num, dim)
Sigma = zeros(dim, dim, num);
Mu = zeros(num, dim);
for k=1:num
 while true
 A = rand(dim, dim);
 if rank(A) == dim; break; end
 end
 Sigma(:, :, k) = A' * A;
 Mu(k, :) = rand(dim, 1);
end
p = abs(rand(num, 1));
p = p / sum(p(:));
gmm = gmdistribution(Mu, Sigma, p);

end
