%________________________________________________________________________-
function likelihood_experiments()
	close all
	clear
	clc
	datasets = {'magic_10', 'wave_21', 'letter_16'};
	out_dir = 'results';
	for idx_dataset=1:1:1
		% repeat experiment for dataset with different random data shufflings
		for exp_itr=1:1:1
			dataset=datasets(idx_dataset);
			evaluate_likelihoods(dataset, exp_itr, out_dir);
		end
	end
end
%________________________________________________________________________-
function evaluate_likelihoods(dataset, exp_itr, out_dir)
	%% load data
	data = load_dataset(dataset);
	data = double(data);
	dim = size(data, 1);
	dim2 = dim;
	ndata = size(data, 2);
	disp(['dataset: ' dataset ', size:' size(data)])

	%% normalize data
	data = data(:, randperm(size(data, 2)));
	mu = mean(data, 2);
	data = bsxfun(@minus, data, mu);
	cv = cov(data');
	data = sqrtm(cv)\data;
	ndata = size(data, 2);
	%% num_coef controls the number of mixture components.
	for num_coef=1:1:2
		%% idx_prior controls the type of prior used.
		for idx_prior=1:1:1
			num = round(num_coef*dim);
			disp(strcat('num: ', num2str(num, '%03d')))
			theta_stc_initial=[];
			%% mixture with stc algorithm for estimation
			D_STC = rmixturefactory(rmvnfactory(dim, dim2), num);
			%% mixture with Newton algorithm for estimation
			D_Newton = rmixturefactory_shared(rmvnfactory_shared(dim, dim2), num);

			options = {};
			options.rho = 0.1; options.lsc = 1e-4; options.lsc2 = 9e-1;
			options.cost_threshold = 1e-8;
			options.grad_threshold = 1e-8;
			options.max_ndata = 100000000;
			options.use_prior = 1;
			options.use_merit = 1;
			options.use_shared = 1;
			options.type_prior = idx_prior;
			mkdir_if_not_exist(out_dir)

			%% apply combination of Newton and STC algorithms
			%% 27 iterations with STC and remaining 73 iterations with Newton.
			options.totalItrNo = 100;
			options.pivot_itr = 27; %0.9* options.totalItrNo;
			log_dir = strcat(out_dir, '/likelihood/exp-', num2str(exp_itr, '%03d'), '__', dataset{1},'__snw__priortype-', num2str(idx_prior), '__coef-', num2str(num_coef, '%02d'), '__dim-', num2str(dim, '%03d'), '__num-' , num2str(num, '%03d'));
			mkdir_if_not_exist(log_dir)
			[theta_Newton, theta0, theta_pivot, theta_set_newton_stc] = estimatelinesearchsharedMAPMixedS(data, D_Newton, options, D_STC, theta_stc_initial, dataset, log_dir);
			theta_file = strcat(log_dir, '/', dataset, '_theta.mat');
			theta_file = string(theta_file);
			save(theta_file, 'theta_set_newton_stc');
			
			%% continue remaining 73 iteration of estimation with STC instead of Newton.
			options.totalItrNo = 100;
			theta_stc_initial = theta_pivot;
			options.totalItrNo = 73; %options.totalItrNo - options.pivot_itr;
			options.pivot_itr = 1+options.totalItrNo;
			log_dir = strcat(out_dir, '/likelihood/exp-', num2str(exp_itr, '%03d'), '__', dataset{1},'__stc__priortype-', num2str(idx_prior), '__coef-', num2str(num_coef, '%02d'), '__dim-', num2str(dim, '%03d'), '__num-' , num2str(num, '%03d'));
			mkdir_if_not_exist(log_dir) 
			[theta_Newton, theta0, theta_pivot, theta_set_stc] = estimatelinesearchsharedMAPMixedS(data, D_Newton, options, D_STC, theta_stc_initial, dataset, log_dir);
			theta_file = strcat(log_dir, '/', dataset, '_theta.mat');
			theta_file = string(theta_file);
			save(theta_file, 'theta_set_stc'); 

			%% apply only Newton algorithm for 100 iterations.
			options.totalItrNo = 100;
			options.pivot_itr = 0;
			log_dir = strcat(out_dir, '/likelihood/exp-', num2str(exp_itr, '%03d'), '__', dataset{1},'__nwt__priortype-', num2str(idx_prior), '__coef-', num2str(num_coef, '%02d'), '__dim-', num2str(dim, '%03d'), '__num-' , num2str(num, '%03d'));
			mkdir_if_not_exist(log_dir) 
			[theta_Newton, theta0, theta_pivot, theta_set_newton] = estimatelinesearchsharedMAPMixedS(data, D_Newton, options, D_STC, theta_stc_initial, dataset, log_dir);
			theta_file = strcat(log_dir, '/', dataset, '_theta.mat');
			theta_file = string(theta_file);
			save(theta_file, 'theta_set_newton');


			%% apply basic EM algorithm on classic GMM for 100 iterations.
			options.totalItrNo = 100;
			d = dim;
			K = round(num_coef*dim);
			P = K - 1 + K*d + d*d + K*d;
			%% the number of compononts is chosen such that numer of parameters remain equal.
			num = round((P + 1) / (1 + d + d*(d + 1) / 2));
			log_dir = strcat(out_dir, '/likelihood/exp-', num2str(exp_itr, '%03d'), '__', dataset{1},'__gmm__priortype-', num2str(idx_prior), '__coef-', num2str(num_coef, '%02d'), '__dim-', num2str(dim, '%03d'), '__num-' , num2str(num, '%03d'));
			mkdir_if_not_exist(log_dir) 
			D = mixturefactory(mvnfactory(dim), num);
			[theta, theta_set_gmm] = gmm_estimation(D, data, num, options, dataset, log_dir);
			theta_file = strcat(log_dir, '/', dataset, '_theta.mat');
			theta_file = string(theta_file);
			save(theta_file, 'theta_set_gmm');
		end    
	end
end