function time_experiments()
	close all
	clear
	clc
	datasets = {'magic_10', 'wave_21', 'letter_16'};
	out_dir = 'results';
	total_itr = 100
	for idx_dataset=1:1:3
			dataset=datasets(idx_dataset);
			measure_times(dataset, out_dir, total_itr);
	end
end
function measure_times(dataset, out_dir, total_itr)
    %% load data
    data = load_dataset(dataset);
    data = double(data);
    dim = size(data,1);
    dim2 = dim;
    ndata = size(data,2);
	disp(['dataset: ' dataset ', size:' size(data)])

    %% normalize data
    data = data(:,randperm(size(data,2)));
    mu = mean(data,2);
    data = bsxfun(@minus,data,mu);
    cv = cov(data');
    data = sqrtm(cv)\data;
    ndata = size(data,2);
        
    %% measuring time complexity
    num_coef = 3;
    num = num_coef*dim;
    theta_stc_initial=[];
    D_STC = rmixturefactory(rmvnfactory(dim,dim2), num);
    D_Newton = rmixturefactory_shared(rmvnfactory_shared(dim,dim2), num);
    options = {};
    options.rho = 0.1; options.lsc = 1e-4; options.lsc2 = 9e-1;
    options.cost_threshold = 1e-8;
    options.grad_threshold = 1e-8;
    options.max_ndata = 10000000;
    options.totalItrNo = 20;
    options.use_prior = 1;
	options.type_prior = 1;
    
	%out_dir = out_dir{1};
	out_dir = strcat(out_dir, '/time');
    mkdir_if_not_exist(out_dir)
    newtonfile = strcat(out_dir, '/newton--', dataset{1}, '.txt');
    stcfile = strcat(out_dir, '/stc--', dataset{1}, '.txt');

    fid = fopen(newtonfile, 'a+');
	text = 'mstep_u';
    fprintf(fid, '%s\t', text);
	text = 'mstep_other_params';
    fprintf(fid, '%s\t', text);
	text = 'mstep_shared_computations';
    fprintf(fid, '%s\t', text);
	text = 'estep';
    fprintf(fid, '%s\n', text);
    fclose(fid);

    fid = fopen(stcfile, 'a+');
	text = 'mstep_u';
    fprintf(fid, '%s\t', text);
	text = 'mstep_other_params';
    fprintf(fid, '%s\t', text);
	text = 'mstep_shared_computations';
    fprintf(fid, '%s\t', text);
	text = 'estep';
    fprintf(fid, '%s\n', text);
    fclose(fid);

    for itr_time=1:total_itr
        options.use_merit = 0;
        options.use_shared = 0;
        options.pivot_itr = 0;
        tic
		time_mstep_u = 0
        [theta_Newton,theta0, theta_pivot, theta_set, time_mstep_u, time_mstep_params, time_mstep_shared, time_estep] = estimatelinesearchsharedMAPMixedS_timing(data,D_Newton,options,D_STC,theta_stc_initial, dataset,out_dir);
        time_mstep_u = num2str(time_mstep_u);
        time_mstep_params = num2str(time_mstep_params);
        time_mstep_shared = num2str(time_mstep_shared);
        time_estep = num2str(time_estep);
        fid = fopen(newtonfile, 'a+');
		text = strcat('itr:',num2str(itr_time), ', merit:', num2str(options.use_merit), ', shared:', num2str(options.use_shared));
		fprintf(fid, '%s\n', text);
		text = time_mstep_u;
		fprintf(fid, '%s\t', text);
		text = time_mstep_params;
		fprintf(fid, '%s\t', text);
		text = time_mstep_shared;
		fprintf(fid, '%s\t', text);
		text = time_estep;
		fprintf(fid, '%s\n', text);
        fclose(fid);

        tic
        options.pivot_itr = options.totalItrNo + 2;
        [theta_STC,theta0, theta_pivot, theta_set, time_mstep_u, time_mstep_params, time_mstep_shared, time_estep] = estimatelinesearchsharedMAPMixedS_timing(data,D_Newton,options,D_STC,theta_stc_initial, dataset,out_dir);
        time_mstep_u = num2str(time_mstep_u);
        time_mstep_params = num2str(time_mstep_params);
        time_mstep_shared = num2str(time_mstep_shared);
        time_estep = num2str(time_estep);

        fid = fopen(stcfile, 'a+');
		text = strcat('itr:',num2str(itr_time), ', merit:', num2str(options.use_merit), ', shared:', num2str(options.use_shared));
		fprintf(fid, '%s\n', text);
		text = time_mstep_u;
		fprintf(fid, '%s\t', text);
		text = time_mstep_params;
		fprintf(fid, '%s\t', text);
		text = time_mstep_shared;
		fprintf(fid, '%s\t', text);
		text = time_estep;
		fprintf(fid, '%s\n', text);
        fclose(fid);
    end

    for itr_time=1:total_itr
        options.use_merit = 0;
        options.use_shared = 1;
        options.pivot_itr = 0;
        tic
        [theta_Newton,theta0, theta_pivot, theta_set, time_mstep_u, time_mstep_params, time_mstep_shared, time_estep] = estimatelinesearchsharedMAPMixedS_timing(data,D_Newton,options,D_STC,theta_stc_initial, dataset,out_dir);
        time_mstep_u = num2str(time_mstep_u);
        time_mstep_params = num2str(time_mstep_params);
        time_mstep_shared = num2str(time_mstep_shared);
        time_estep = num2str(time_estep);
        fid = fopen(newtonfile, 'a+');
		text = strcat('itr:',num2str(itr_time), ', merit:', num2str(options.use_merit), ', shared:', num2str(options.use_shared));
		fprintf(fid, '%s\n', text);
		text = time_mstep_u;
		fprintf(fid, '%s\t', text);
		text = time_mstep_params;
		fprintf(fid, '%s\t', text);
		text = time_mstep_shared;
		fprintf(fid, '%s\t', text);
		text = time_estep;
		fprintf(fid, '%s\n', text);
        fclose(fid);

        options.pivot_itr = options.totalItrNo + 2;
        tic
        [theta_STC,theta0, theta_pivot, theta_set, time_mstep_u, time_mstep_params, time_mstep_shared, time_estep] = estimatelinesearchsharedMAPMixedS_timing(data,D_Newton,options,D_STC,theta_stc_initial, dataset,out_dir);
        time_mstep_u = num2str(time_mstep_u);
        time_mstep_params = num2str(time_mstep_params);
        time_mstep_shared = num2str(time_mstep_shared);
        time_estep = num2str(time_estep);
        fid = fopen(stcfile, 'a+');
		text = strcat('itr:',num2str(itr_time), ', merit:', num2str(options.use_merit), ', shared:', num2str(options.use_shared));
		fprintf(fid, '%s\n', text);
		text = time_mstep_u;
		fprintf(fid, '%s\t', text);
		text = time_mstep_params;
		fprintf(fid, '%s\t', text);
		text = time_mstep_shared;
		fprintf(fid, '%s\t', text);
		text = time_estep;
		fprintf(fid, '%s\n', text);
        fclose(fid);
    end
    for itr_time=1:total_itr
        options.use_merit = 1;
        options.use_shared = 0;
        options.pivot_itr = 0;
        tic
        [theta_Newton,theta0, theta_pivot, theta_set, time_mstep_u, time_mstep_params, time_mstep_shared, time_estep] = estimatelinesearchsharedMAPMixedS_timing(data,D_Newton,options,D_STC,theta_stc_initial, dataset,out_dir);
        time_mstep_u = num2str(time_mstep_u);
        time_mstep_params = num2str(time_mstep_params);
        time_mstep_shared = num2str(time_mstep_shared);
        time_estep = num2str(time_estep);
        fid = fopen(newtonfile, 'a+');
		text = strcat('itr:',num2str(itr_time), ', merit:', num2str(options.use_merit), ', shared:', num2str(options.use_shared));
		fprintf(fid, '%s\n', text);
		text = time_mstep_u;
		fprintf(fid, '%s\t', text);
		text = time_mstep_params;
		fprintf(fid, '%s\t', text);
		text = time_mstep_shared;
		fprintf(fid, '%s\t', text);
		text = time_estep;
		fprintf(fid, '%s\n', text);
        fclose(fid);
    end
    for itr_time=1:total_itr
	disp('fourth')
        options.use_merit = 1;
        options.use_shared = 1;
        options.pivot_itr = 0;
        tic
        [theta_Newton,theta0, theta_pivot, theta_set, time_mstep_u, time_mstep_params, time_mstep_shared, time_estep] = estimatelinesearchsharedMAPMixedS_timing(data,D_Newton,options,D_STC,theta_stc_initial, dataset,out_dir);
        time_mstep_u = num2str(time_mstep_u);
        time_mstep_params = num2str(time_mstep_params);
        time_mstep_shared = num2str(time_mstep_shared);
        time_estep = num2str(time_estep);
        fid = fopen(newtonfile, 'a+');
		text = strcat('itr:',num2str(itr_time), ', merit:', num2str(options.use_merit), ', shared:', num2str(options.use_shared));
		fprintf(fid, '%s\n', text);
		text = time_mstep_u;
		fprintf(fid, '%s\t', text);
		text = time_mstep_params;
		fprintf(fid, '%s\t', text);
		text = time_mstep_shared;
		fprintf(fid, '%s\t', text);
		text = time_estep;
		fprintf(fid, '%s\n', text);
        fclose(fid);
    end
end
