function [ out ] = bldiagmult( cellblock,M,lr,tryfull,sparseflag )
%Save block diagonal matrix B as cell array, i.e. B = blkdiag(cellblock{:})
% is saved as a number of cells ("cells" in this code) containing a matrix
% Multiply with matrix / vector  "M"
% if option lr = 'l' (default) have  B * M 
% else lr = 'r'  M * B

%to perform multiplations with the transpose of B, eg M * B' do (B * M')'
%  bldiagmult( cellblock',M,'r' )  = (bldiagmult( cellblock,M','l' )) '
% input M can be sparse as can cellblock, but cellblock should be full in
% order to achieve a speed up

%tested to work
%sparseflag given makes the output "out" a sparse matrix, can be anything
%tryfull if set to one will only have an impact if the input is sparse,
%this will mean that it will try to use full matrix multiplication at some
%stages (when possible), this will not always be faster

%this routine will likely be slower than the inbuilt matrix multiplication
%in matlab unless B is very large.
%This function can also be used when B = blkdiag(cellblock{:}) would not 
%fit in the memory

%David Holdaway 2012

sparsM = issparse(M);

if nargin ==2
    lr = 'l';
end
if nargin <4
    tryfull = false;
end
sz2 = size(M);
cells = length(cellblock); sz =zeros(cells,2);
for k = 1:cells
sz(k,:)= size(cellblock{k}); 
end
szsum = [0,0;cumsum(sz,1)]; %first one is zero to fit with later logic
if strcmp(lr,'l')==1  % B * M 
    %size of out will be #rows in B # columns in M

%test that multiplication is legal
if sz2(1) ~=szsum(cells+1,2)
    [sz2(1),szsum(cells+1,2) ] %error in block size
    error('matricies cannot be multiplied, size mismatch')

end

if nargin < 5
out = zeros(szsum(cells+1,1),sz2(2));
else
out = sparse(szsum(cells+1,1),sz2(2)); 
end



for k =1:cells
   
    rng1 = szsum(k,1)+1:szsum(k,1)+sz(k,1);
    rng2 = szsum(k,2)+1:szsum(k,2)+sz(k,2);
   if isempty(rng2)==0 && isempty(rng1)==0
       
       if sparsM && tryfull
    try
    out(rng1,:) = cellblock{k}*full(M(rng2,:)); %if this gives mem issues
    catch 
    out(rng1,:) = cellblock{k}*M(rng2,:);    %use sparse multiplication
    end
       else
    out(rng1,:) = cellblock{k}*M(rng2,:); %no need to worry        
           
       end

   end    
    
end

elseif  strcmp(lr,'r')==1    %  M * B
       %size of out will be #rows in M # columns in B

if sz2(2) ~=szsum(cells+1,1)
       [sz2(2),szsum(cells+1,1) ]
    error('matricies cannot be multiplied, size mismatch')

end

if nargin < 4
out = zeros(sz2(1),szsum(cells+1,2));
else
out = sparse(sz2(1),szsum(cells+1,2));
end


for k =1:cells

 
    rng1 = szsum(k,1)+1:szsum(k,1)+sz(k,1);
    rng2 = szsum(k,2)+1:szsum(k,2)+sz(k,2);
   if isempty(rng2)==0 && isempty(rng1)==0
    
    if sparsM && tryfull
    try
        out(:,rng2) = full(M(:,rng1))*cellblock{k};
    catch
        out(:,rng2) = M(:,rng1)*cellblock{k}; 
    end
    else
        out(:,rng2) = M(:,rng1)*cellblock{k};
    end
   end
end
else
    error('third argument should be character l or r, no output given')
end

end

%to test try with matricies you want to use or say
% for k = 1:100
% cellbl{k} = rand(80,2);
% end
% M = rand(200);

% outcomp = blkdiag(cellblock{:})*M;
% out = bldiagmult( cellblock,M,'l' );
%all(all(outcomp == out)) %should give unity (does for example), or 
% all(all(abs(outcomp - out)<eps*abs(max(outcomp)))) %at least should be
% true
