function [theta, theta0, theta_pivot, theta_set, time_mstep_u, time_mstep_params, time_mstep_shared, time_estep] = estimatelinesearchsharedMAPMixedS(data, D_newton, options, D_stc, theta_stc_initial, dataset,directory, theta_initial)
time_mstep_u = 0;
time_mstep_params = 0;
time_mstep_shared = 0;
time_estep = 0;
theta_set = {};
theta_pivot = [];
D = D_newton;
log_text = '';
%% declare parameters
data = mxe_readdata(data, false);
dim = data.dim;
ndata = data.size;
num = D.num();
%% init model parameters
theta = init_model_params(D, data, options);
if (options.pivot_itr == 1+options.totalItrNo)
	theta = theta_stc_initial;
end
if(nargin > 7 )
    disp('theta_initial')
    theta = theta_initial;
end
theta0 = theta;
%% init prior parameters
[thetap, store] = init_prior_params(D, theta, data, options);
if (options.use_shared == 1)
	store.M = shared_matrix_fast(D, data, store, options);
else
    store.M = shared_matrix(num,theta,data, dim, store, options);
end

%% EM loop
llvec = zeros(1, options.totalItrNo);
llgvec = zeros(1, options.totalItrNo);

%% First call to cost and gradient functions
[f0, g0, f_ll0] = cost_gradient(D, theta, data, store, options);
itr = 1; llvec(itr) = f0; llgvec(itr) = norm(g0, 'fro');
str = display_message(itr, f0, g0, f_ll0);
log_text = strcat(log_text, str);
%% EM loop
theta_set{1} = theta;
itr = 0;
is_converged = 0;

store.component_weights = D.weighting(theta, data, store);
store = shared_computations_6(data, store);
store = shared_computations_5(store);
if (options.use_shared == 1)
    store.M = shared_matrix_fast(D, data, store, options);
else
    store.M = shared_matrix(num,theta,data, dim, store, options);
end
while (itr < options.totalItrNo && is_converged == 0)
    itr = itr+1;
    if (itr <= options.pivot_itr)
        D = D_stc;
    else
        D = D_newton;
    end
    %% M-step: update parameters
tic_total = tic;
    [theta, store] = update_u(D_newton, D_stc, theta, data, store, options, itr);
time_mstep_u = time_mstep_u + toc(tic_total);

tic_total = tic;
    store = shared_computations_1(theta, data, store);
%     store = shared_computations_2(theta, D, dim, store);
    store = shared_computations_3(theta, D, store);
    store = shared_computations_4(D, store);
    store = shared_computations_5(store);
%     store = shared_computations_6(data, store);
time_mstep_shared = time_mstep_shared + toc(tic_total);

    %[f1, g1, f_ll1] = cost_gradient(D_newton, theta, data, store, options);
    tic_total = tic;
    [theta, store] = update_mu(D, theta, data, store, options, num);
    %[f3, g3, f_ll3] = cost_gradient(D_newton, theta, data, store, options);    
    [theta, store] = update_d(D, theta, store, options, num, dim);    
    %[f4, g4, f_ll4] = cost_gradient(D_newton, theta, data, store, options);    
    [theta, store] = update_p(D, theta, thetap, store, options, ndata);
    time_mstep_params = time_mstep_params + toc(tic_total);

    if (options.use_shared == 1)
        store.M = shared_matrix_fast(D, data, store, options);
    else
        store.M = shared_matrix(num,theta,data, dim, store, options);
    end

    if(itr==10)
        disp('');
    end
    [f2, g2, f_ll2] = cost_gradient(D_newton, theta, data, store, options);        
    str = display_message(itr, f2, g2, f_ll2);
    log_text = strcat(log_text , '\r\n' , str);    
    if (itr == options.pivot_itr)
    	theta_pivot = theta;
    end

tic_total = tic;
	store.component_weights = D.weighting(theta, data, store);
time_estep = time_estep + toc(tic_total);

tic_total = tic;
    store = shared_computations_1(theta, data, store);
    store = shared_computations_2(theta, D, dim, store);
    store = shared_computations_3(theta, D, store);
    store = shared_computations_4(D, store);
    store = shared_computations_5(store);
    store = shared_computations_6(data, store);
    if (options.use_shared == 1)
        store.M = shared_matrix_fast(D, data, store, options);
    else
        store.M = shared_matrix(num,theta,data, dim, store, options);

    end
time_mstep_shared = time_mstep_shared + toc(tic_total);
    

	theta_set{itr} = theta;
end % of itr loop
write_log(dataset, log_text, directory)
end % of estimatelinesearch function
%-------------------------------------------------------------------------
function [theta, store] = update_u(D_newton, D_stc, theta, data, store, options, itr)
if (itr <= options.pivot_itr)
    [theta, store] = update_u_stc(D_stc, theta, data, store, options);
else
    [theta, store] = update_u_newton(D_stc, theta, data, store, options);
end
end
%-------------------------------------------------------------------------
function [theta, store] = update_u_stc(D, theta, data, store, options)
%         D = D_stc;
dim = D.datadim();
if(options.use_prior == 1)
    beta = sum(store.component_weights(:)) + store.nup - dim - 1;
else
    beta = sum(store.component_weights(:));
end
cof = cofactor_stc(theta.U);
for i = 1:dim
    index = (i-1)*dim+1;
    Gi = store.M(:,index:index+dim-1);
    ci = cof(:,i);
    ai = sqrt(beta/(ci'*(Gi\ci)))*(Gi\ci);
    theta.U(:,i)=ai;
end
storeNew = store;
storeNew = shared_computations_1(theta, data, storeNew);
storeNew = shared_computations_3(theta, D, storeNew);
storeNew = shared_computations_4(D, storeNew);
storeNew = shared_computations_5(storeNew);
store = storeNew;
end
%-------------------------------------------------------------------------
function [theta, store] = update_u_newton(D, theta, data, store, options)
K_newton=1;
ndata = data.size();
dim = D.datadim();
for k_newton=1:K_newton
    tic_g = tic();
    g = gradient(data, ndata, dim, theta, store, options);            
    time_g = toc(tic_g);
    tic_g = tic();
    hmat = descent_direction(D, data, theta, ndata, dim, g, store, options);
    time_descent = toc(tic_g);
    %% Line search: step-length for U
    tic_g = tic();
    [fmOld, fmNew, thetaNew] = armijo_linesearch(D, theta, hmat, data, g, store, options);
    time_armijo = toc(tic_g);
    tic_g = tic();
    %storeNew = store;
    %storeNew = shared_computations_1(thetaNew, data, storeNew);
    %storeNew = shared_computations_3(thetaNew, D, storeNew);
    %storeNew = shared_computations_4(D, storeNew);
    %storeNew = shared_computations_5(storeNew);
    %store = storeNew;
    theta = thetaNew;
    time_shared = toc(tic_g);
disp(time_g)
disp(time_descent)
disp(time_armijo)
disp(time_shared)
end
end
%-------------------------------------------------------------------------
function [theta, store] = update_mu(D, theta, data, store, options, num)
    tic_mu = tic;
    mu = store.component_weights*data.data';
    if(options.use_prior==1)
        mu = bsxfun(@plus, mu, store.kp*store.mup');
        mu = bsxfun(@times, mu , 1./(store.kp+store.sumW));
    else
        mu = bsxfun(@times, mu , 1./store.sumW);
    end
    for k = 1:num
        theta.D{k}.mu = mu(k,:)';
    end
    time_6_update_mu = toc(tic_mu);
    
    %% Update store
    store = shared_computations_3(theta, D, store);
    store = shared_computations_4(D, store);
    store = shared_computations_5(store);
    %     [f_mu, g_mu] = cost_gradient_ll(D, theta, data, store, options);
end
%-------------------------------------------------------------------------
function [theta, store] = update_d(D, theta, store, options, num, dim)
num = D.num();
dim = D.datadim();
for k = 1:num
    sumw_k = store.sumW(k);
    storec = store.componentStores{k};
    for j = 1:dim
        if(options.use_prior==1)
            temp2 = (store.nup-2+sumw_k)/(store.sp+store.WXU2(k, j)+storec.WMU2(j)-2*store.WXU(k, j)*storec.MU(j));
        else
            temp2 = (sumw_k)/(store.WXU2(k, j)+storec.WMU2(j)-2*store.WXU(k, j)*storec.MU(j));
        end
        theta.D{k}.d(j) = temp2;
    end
% Update store
store = shared_computations_2(theta, D, dim, store);
end
end
%-------------------------------------------------------------------------
function [theta, store] = update_p(D, theta, thetap, store, options, ndata)
if (options.use_prior==1)
	theta.p = (thetap.ap-1+store.sumW)/(ndata+sum(thetap.ap-1));
else
	theta.p = store.sumW/ndata;
end
end
%-------------------------------------------------------------------------
function theta = init_model_params(D, data, options)
dim = data.dim;
ndata = data.size;
num = D.num();

idx = 1:ndata;
idx = idx(randperm(ndata));
ndata_initial = ndata;
data_initial = data.data(:, idx);
data_initial = data_initial(:, 1:ndata_initial);
data_initial = mxe_readdata(data_initial, false);
%% initialize parameters
theta.p = ones(num, 1) ./ num;
Dmvn = mvnfactory(dim);
theta0 = Dmvn.estimate(data_initial);
[theta.U, ~] = eig(theta0.sigma);
% theta.U = eye(dim);
label = ceil(rand(1, ndata_initial) * num);
theta.D = cell(num, 1);
for k = 1:num
    theta0 = Dmvn.estimate(data_initial.data(:, label == k));
    theta.D{k}.mu = theta0.mu;
    theta.D{k}.d = eig(theta0.sigma);
end
end
%-------------------------------------------------------------------------
function [thetap, store] = init_prior_params(D, theta, data, options)
dim = data.dim;
ndata = data.size;
num = D.num();
thetap = struct;
if(options.use_prior == 1)
    thetap.mup = sum(data.data, 2)/ndata;
    thetap.kp = 0.01;
    thetap.nup = dim+2;
options.type_prior
    %% scenario 1
    if(options.use_prior == 1)
        thetap.Sp = cov(data.data')/(num^(2/dim));
        thetap.sp = sum(diag(cov(data.data')))/dim/(num^(2/dim));
        %thetap.sp = sum(diag(cov(data.data')))/(num^(2/dim));
    end
    %% scenario 2
    if(options.use_prior == 2)
        thetap.Sp = cov(data.data')/(num^(2/dim));
        thetap.sp = sum(diag(cov(data.data')))/dim;
    end
    %% scenario 3
    if(options.use_prior == 3)
        thetap.Sp = cov(data.data')/(1);
        thetap.sp = sum(diag(cov(data.data')))/dim/(num^(2/dim));
    end
    %% scenario 4
    if(options.use_prior == 4)
        thetap.Sp = cov(data.data')/(1);
        thetap.sp = sum(diag(cov(data.data')))/dim;
    end
    %% inverse of Sp
    thetap.Spi = inv(thetap.Sp);
    thetap.ap = ones(num, 1)+1e-2;
end

store = thetap;
store = shared_computations_1(theta, data, store);
store = shared_computations_2(theta, D, dim, store);
store = shared_computations_3(theta, D, store);
store.component_weights = D.weighting(theta, data, store);
store = shared_computations_4(D, store);
store = shared_computations_5(store);
store = shared_computations_6(data, store);

P = zeros(dim^2);
for i=1:dim
    for j=1:dim
        P((i-1)*dim+j,i+(j-1)*dim)=1;
    end
end
store.P = P;

end
%-------------------------------------------------------------------------
function [fmOld, fmNew, thetaNew] = armijo_linesearch(D, theta, H, data, g, store, options)
[dim, ndata] = size(data.data);
hvec = H(:);
%g = gradient(data, ndata, dim, theta, store, options);
gvec = g(:);
if(options.use_merit == 1)
    fmOld = norm(g, 'fro')/2;
    r = g;
    g1 = DfH(theta, data, ndata, dim, r, store, options);
    gvec = g1(:);
else
    fmOld = cost_gradient(D, theta, data, store, options);
end
a = 10;
while true
    thetaNew = theta;
    thetaNew.U = theta.U+a*H;
    storeNew = store;
    %storeNew = shared_computations_1(thetaNew, data, storeNew);
    %storeNew = shared_computations_3(thetaNew, D, storeNew);
    %storeNew = shared_computations_4(D, storeNew);
    %storeNew = shared_computations_5(storeNew);

    if(options.use_merit == 1)
        rNew = gradient(data, ndata, dim, thetaNew, storeNew, options);
        fmNew = norm(rNew, 'fro')/2;
    else
        fmNew = cost_gradient(D, thetaNew, data, storeNew, options);
%         precision = test_cost_ll(D, thetaNew, data, storeNew, options);
%         disp(strcat('ll precision: ', num2str(precision)));
    end
    
    if(fmNew<= fmOld+options.lsc*a*gvec'*hvec)
        disp(strcat('fmNew: ',num2str(fmNew)))
        disp(strcat('a: ', num2str(a)))
        return;
    end
    a = a*options.rho;
end
end
%-------------------------------------------------------------------------
function [f, g, f_ll] = cost_gradient(D, theta, data, store, options)
ndata = data.size();
dim = data.dim();
f = -D.ll(theta, data, store)/ndata;
f_ll = f;
if(options.use_prior == 1)
    lp = -logprior(D, theta, store)/ndata;
    f = f + lp;
end
if nargout < 2
    return;
end
g = gradient(data, ndata, dim, theta, store, options);

% num = D.num();
% g0 = gradient_dummy(num,theta,data, dim, store, options);
% message = strcat(num2str(max(max(g-g0))), ',');
% directory = '.';
% dataset = 'magic_10';
% write_log(dataset, message, directory)
% test_cost_ll(D, theta, data, store, options);
end
%-------------------------------------------------------------------------
function g = gradient(data, ndata, dim, theta, store, options)
alpha = ndata;
g = optimized_product(store.M, theta.U)';
if(options.use_prior == 1)
    g = g + store.Spi * theta.U;
    alpha = ndata+store.nup-dim-2;
end
g = g-alpha*(theta.U'\eye(dim));
g = g / alpha;
end
%-------------------------------------------------------------------------
function [g] = DfH(theta, data, ndata, dim, H, store, options)
alpha = ndata;
g = optimized_product(store.M, H)';
if(options.use_prior == 1)
    g = g + store.Spi*H;
    alpha = ndata+store.nup-dim-2;
end
g = g + alpha*(theta.U'\H')*(theta.U'\eye(dim));
g = g / alpha;
end
%-------------------------------------------------------------------------
function [g] = DfH2(theta, data, ndata, dim, H, store, options)
    H = vec2mat(H,dim)';
    g = DfH(theta, data, ndata, dim, H, store, options);
    g = g(:);
end
%-------------------------------------------------------------------------
function P = descent_direction(D, data, theta, ndata, dim, g, store, options)
flag = 0;
if flag == 0
    rtol = 1e-10;   maxit = dim;
    A = @(H)DfH2(theta, data, ndata, dim, H, store, options);
    b = -g(:);
    p = minresqlp3(A,b,rtol,maxit);
    P = vec2mat(p,dim)';
else
    tic_g = tic();
    if (options.use_shared == 1)
        [A,b] = get_jacobian_shared(D, theta, data, g, store, options);
    else
        [A,b] = get_jacobian(D, theta, data, store, options);
    end
    time_j = toc(tic_g);
    tic_g = tic();
    p = A\(-b);
    time_p1 = toc(tic_g);
    tic_g = tic();
    P = vec2mat(p,dim)';
    time_p2 = toc(tic_g);
    disp('------------------------')
    disp(time_j)
    disp(time_p1)
    disp(time_p2)
    disp('------------------------')
end
return;
end
%-------------------------------------------------------------------------
function [A,b] = get_jacobian(D, theta, data, store, options)
ndata = data.size();
dim = D.datadim();
num = D.num();
X = data.data();
alpha = ndata;


S2 = zeros(dim,dim);
gamma = store.component_weights;
for k=1:num
    %Ak = store.kp * (theta.D{k}.mu-store.mup) * (theta.D{k}.mu-store.mup)';
    muk = theta.D{k}.mu;
    if(options.use_prior==1)
        Ak = store.kp * (theta.D{k}.mu-store.mup) * (theta.D{k}.mu-store.mup)';
    else
        Ak = zeros(dim);
    end
    gamma_k = sqrt(gamma(k,:));
    Ak0 = bsxfun(@plus, X, -muk);
    Ak0 = bsxfun(@times, Ak0 , gamma_k);
    Ak = Ak0 * Ak0';
    S2 = S2 + theta.U'*Ak*theta.U*diag(theta.D{k}.d);
end
S2 = S2 + theta.U' * store.Spi * theta.U - alpha * eye(dim);


X = data.data();
gamma = store.component_weights;
S3 = zeros(dim^2,dim^2);
S4 = zeros(dim,dim);
for k=1:num
    %Ak = store.kp * (theta.D{k}.mu-store.mup) * (theta.D{k}.mu-store.mup)';
    muk = theta.D{k}.mu;
    if(options.use_prior==1)
        Ak = store.kp * (theta.D{k}.mu-store.mup) * (theta.D{k}.mu-store.mup)';
    else
        Ak = zeros(dim);
    end
    gamma_k = sqrt(gamma(k,:));
    Ak0 = bsxfun(@plus, X, -muk);
    Ak0 = bsxfun(@times, Ak0 , gamma_k);
    Ak = Ak0 * Ak0';
    S3 = S3 + kron(diag(theta.D{k}.d),theta.U'*Ak);
    S4 = S4 + Ak*theta.U*diag(theta.D{k}.d);
end

A1 = S3;


A = theta.U'*store.Spi;
A2 = kron(eye(dim),A);



A = S4 + store.Spi*theta.U;
C = kron(eye(dim),A');
A3 = zeros(size(C));
for i=1:dim
    for j=1:dim
        k = (i-1)*dim+j;
        k2 = (j-1)*dim+i;
        A3(k,:)= C(k2,:);
    end
end
A = A1+A2+A3;
b = S2(:);
A = A / alpha;
b = b / alpha;
end
%-------------------------------------------------------------------------

function M = shared_matrix_fast(D, data, store,options)
num = D.num();
dim = data.dim();
X = data.data();

S1 = (store.D*store.component_weights)*(kron(ones(dim, 1), X).*kron(X, ones(dim, 1)))';
S2 = (kron(ones(dim, 1), store.Mu).*kron(store.Mu, ones(dim, 1)))';
S3 = (kron(ones(dim, 1), store.XW).*kron(store.Mu, ones(dim, 1)))';
S4 = (kron(ones(dim, 1), store.Mu).*kron(store.XW, ones(dim, 1)))';
M1 = S1 - store.D*(S3 + S4) + (store.D .* repmat(store.sumW', dim, 1))*S2;
M1 = M1';
M1 = reshape(M1, dim, dim^2);
M = M1;
if(options.use_prior==1)
    Mp = kron(ones(1,num),store.mup);
    S5 = (kron(ones(dim, 1), store.Mu).*kron(store.Mu, ones(dim, 1)))';
    S6 = (kron(ones(dim, 1), Mp).*kron(Mp, ones(dim, 1)))';
    S7 = (kron(ones(dim, 1), store.Mu).*kron(Mp, ones(dim, 1)))';
    S8 = (kron(ones(dim, 1), Mp).*kron(store.Mu, ones(dim, 1)))';
    M2 = store.kp*store.D*(S5+S6-S7-S8);
    M2 = M2';
    M2 = reshape(M2, dim, dim^2);
    M = M + M2;
end
end
%-------------------------------------------------------------------------
function M = shared_matrix(num,theta,data, dim, store, options)
ndata = data.size();
S = zeros(dim,dim^2);
X = data.data();
gamma = store.component_weights;
for k=1:num
    muk = theta.D{k}.mu;
    if(options.use_prior==1)
        Ak = store.kp * (theta.D{k}.mu-store.mup) * (theta.D{k}.mu-store.mup)';
    else
        Ak = zeros(dim);
    end
    gamma_k = sqrt(gamma(k,:));
    Ak0 = bsxfun(@plus, X, -muk);
    Ak0 = bsxfun(@times, Ak0 , gamma_k);
    Ak = Ak0 * Ak0';
%     for i=1:ndata
%         xi = X(:,i);
%         Ak = Ak +  gamma(k,i)*xi*(xi');
%         Ak = Ak + gamma(k,i)*muk*(muk');
%         Ak = Ak - gamma(k,i)*xi*(muk');
%         Ak = Ak - gamma(k,i)*muk*(xi');
%     end
    S = S + kron(theta.D{k}.d', Ak);
end
M = S;
end
%-------------------------------------------------------------------------
function [A,b] = get_jacobian_shared(D, theta, data, g, store, options)
ndata = data.size();
dim = D.datadim();
if(options.use_prior == 1)
    alpha = ndata+store.nup-dim-2;
else
    alpha = ndata;
end

%b = gradient(data, ndata, dim, theta, store, options);
b = g(:);

% S44 = optimized_product(store.M, theta.U)';
% S44 = zeros(size(S44));

% M1 = kron_diag(eye(dim),theta.U');
M1 = kron_diag(eye(dim),eye(dim));
A11 = zeros(dim^2);
for d=1:dim
    is = (d-1) * dim + 1;
    ie = is + dim - 1;
    A11(is:ie,is:ie) = M1(:,is:ie)*store.M(:,is:ie);
end

% A = theta.U'*store.Spi;
if(options.use_prior==1)
    A = store.Spi;
    A22 = kron(eye(dim),A);
else
    A22 = zeros(dim^2);
end



% A33 = S44 + store.Spi*theta.U;
% C = kron(eye(dim),A33');
% A33 = store.P*C;

iU = inv(theta.U);
K=commutation(dim);
A33 = kron(iU,iU')*K;

A = A11+A22+alpha*A33;
A = A / alpha;
% b = b / alpha;
end
%-------------------------------------------------------------------------
function [store] = shared_computations_1(theta, data, store)
% stores XU and logdetU
if nargin < 3
    store = struct;
end
store.XU = data.data'*theta.U;
store.XU2 = store.XU.^2;
store.IU = inv(theta.U);
Y = lu(theta.U);
store.logdetU = sum(log(abs(diag(Y))));
end
%-------------------------------------------------------------------------
function [store] = shared_computations_2(theta, D, dim, store)
% stores component variable logdetD
num = D.num();
store.D = zeros(dim, num);
for k = 1:num
    thetac = theta.D{k};
    store.D(:, k) = thetac.d;
    store.componentStores{k}.logdetD = sum(log(thetac.d));
end
end
%-------------------------------------------------------------------------
function [store] = shared_computations_3(theta, D, store)
num = D.num();
dim = D.datadim();
store.Mu = zeros(dim, num);
for k = 1:num
    thetac = theta.D{k};
    store.Mu(:, k) = thetac.mu;
    store.componentStores{k}.MU = thetac.mu'*theta.U;
    store.componentStores{k}.MU2 = store.componentStores{k}.MU.^2;
end
end
%-------------------------------------------------------------------------
function [store] = shared_computations_5(store)
% stores XU and logdetU
if nargin < 1
    store = struct;
end
store.WXU = store.component_weights*store.XU;
store.WXU2 = store.component_weights*store.XU2;
end
%-------------------------------------------------------------------------
function [store] = shared_computations_4(D, store)
num = D.num();
for k = 1:num
    store.componentStores{k}.WMU2 = sum(store.component_weights(k, :))*store.componentStores{k}.MU2;
end
end
%-------------------------------------------------------------------------
function [store] = shared_computations_6(data, store)
if nargin < 2
    store = struct;
end
store.weights = sqrt(store.component_weights);
store.sumW = sum(store.component_weights, 2);
store.XW = data.data* (store.component_weights');
end
%-------------------------------------------------------------------------
function lp = logprior(D, theta, store)
num = D.num();
dim = D.datadim();

UU = theta.U*theta.U';
a = store.nup/2;
B = store.Spi;
inverse = 0;
lp1 = wishpdfln(UU, a, B, inverse);

% p1 = wishpdf(UU, a, B, inverse);
% nopositivedefinite = any(eig(UU) < 0);
% if(nopositivedefinite == 1)
% disp('CHECK');
% end
%

store.ap = zeros(num, 1)+1e-2;
p2 = dirpdf(theta.p, store.ap);
lp2 = log(p2);

a = store.nup/2;
b = store.sp/2;
lp3 = 0;
for k = 1:num
    d = theta.D{k}.d;
%     disp(strcat(num2str(min(d)), ',', num2str(max(d))));
    lp3 = lp3+ sum(log(gampdf(d, a, b)));
end

psd_flag = 0;
sym_flag = 0;
lp4 = 0;
lp5 = 0;
for k = 1:num
    mu = theta.D{k}.mu;
    dk = theta.D{k}.d;
    Dk = diag(theta.D{k}.d);
    y = 0;
    y = y + 0.5*(-dim*log(2*pi)+dim*log(store.kp)+2*store.logdetU+store.componentStores{k}.logdetD);
    s = theta.U'*(mu-store.mup);
    s = sqrt(dk).*s;
    s = -store.kp/2*((s')*s);
    y = y + s;
    lp4 = lp4 + y;
    
    Sx = inv(theta.U*Dk*(theta.U'));
    Sx = 0.5*(Sx+Sx');
    if(any(eig(Sx) < 0))
        psd_flag = 1;
    end
    if(issymmetric(Sx) == 0)
        sym_flag = 1;
    end
    % sym_flag = issymmetric(Sx);
    if(psd_flag == 1)
        disp('CHECK PSD');
    end
    if(sym_flag == 1)
        disp('CHECK SYM');
    end
    lp5 = lp5 + log(mvnpdf(mu, store.mup, Sx/store.kp));
    
end
lp = lp1 + lp2 + lp3 + lp4;
end
%-------------------------------------------------------------------------
function B = kron_diag(D,A)
d = diag(D)';
B = kron(d,A);
end
%-------------------------------------------------------------------------
function Y = optimized_product(M, X)
dim = size(X, 1);
X2 = repmat(X, 1, dim);
% tic
Y = zeros(dim, dim);
% disp(num2str(size(M)))
for i = 1:dim
    istart = (i-1)*dim+1;
    iend = istart + dim -1;
    Y(:, i) = M(:, istart:iend)*X(:, i);
end
Y = Y';

% toc
% tic
% M2 = reshape(M, dim^2, dim);
% X2 = X(:);
% X2 = repmat(X2, 1, dim);
% Y2 = M2.*X2;
% Y2 = sum(Y2, 2);
%
% X2 = kron(X, diag(dim));
% Y2 = M.*X2;
% Y2 = reshape(Y2, dim, dim);
% Y2 = Y2';
% toc
end
%-------------------------------------------------------------------------
function cof = cofactor_stc(a)
[r c] = size(a);    %determine size of input           
m = ones(r,c);      %preallocate r x c cofactor matrix        
a_temp=a;           %create temporary matrix equal to input
for i = 1:r
    for k = 1:c
        a_temp([i],:)=[];   %remove ith row
        a_temp(:,[k])=[];   %remove kth row
        m(i,k) = ((-1)^(i+k))*det(a_temp);  %compute cofactor element
        a_temp=a;   %reset elements of temporary matrix to input elements
    end  
end
cof=m;  %return cofactor matrix as output variable
end
%-------------------------------------------------------------------------
function mkdir_if_not_exist(dirpath)
    if dirpath(end) ~= '/', dirpath = [dirpath '/']; end
    if (exist(dirpath, 'dir') == 0), mkdir(dirpath); end
end
%-------------------------------------------------------------------------
function write_log(filename, text,foldername)
disp(text)
filename = string(filename);
% foldername = string(foldername);
mkdir_if_not_exist(foldername)
filename = strcat(foldername,'/', filename,'.txt');
%open file with write permission
fid = fopen(filename, 'a');
%write a line of text
fprintf(fid, '%s\n', text);
%close filefprintf newlinefprmatla
fclose(fid);
end%-------------------------------------------------------------------------
%-------------------------------------------------------------------------
function message = display_message(itr, f, g, f_ll)
message = strcat(num2str(itr), ':', ', ',num2str(round(f_ll,4)), ',',num2str(round(f,4)), ',', num2str(norm(g, 'fro')));
% message = strcat(num2str(itr), ':', ', ', num2str(round(f_ll,4)), ', ', num2str(norm(g, 'fro')));
disp(message);
str = '----------------------------------------------------------------';
disp(str);
end
%-------------------------------------------------------------------------
function K=commutation(m,n,fast)
% H1 line
%
% Syntax
% -------
% ::
%
% Inputs
% -------
%
% Outputs
% --------
%
% More About
% ------------
%
% Examples
% ---------
%
% See also:

% properties of the commutation matrix
%                i) Kmn'=Knm
%                ii) inv(Kmn)=Knm
%                iii) K1n=Kn1=In
%                iv) trace(Kmn)=1+d(m-1,n-1) where d(x,y) is the greatest
%                common divisor of x and y
%                v) det(Kmn)=(-1)^(0.25*m*(m-1)*n*(n-1))
%                vi) Kmn*vec(A)=vec(A')
%                vii) Kmn*(kron(A,B))*Kst =kron(B,A), where A is (n,s) and
%                B is (m,t). Equivalently, Kmn*(kron(A,B)) =kron(B,A)*Kts
%                viii) Let P be a matrix with m rows, Q a matrix with n
%                      rows, x an m-vector and y an n-vector and z
%                      arbitrary. We have
%                      kron(kron(z',P),y)=Kmn*kron(y*z',P)
%                      kron(kron(x,Q),z')=Kmn*kron(Q,x*z')
%                ix) trace(Kmn*kron(A',B)=trace(A'B)=vec(A')'*Kmn*vec(B)
%                where A and B are (m,n) matrices
if nargin<3
    fast=[];
    if nargin<2
        n=[];
    end
end
if isempty(fast)
    fast=true;
end
if isempty(n)
    n=m;
end

K=zeros(m*n);

if fast
    if fast==2
        nm=n*m;
        J=(1:n).';
        IND=nan(nm,2);
        offset=0;
    end
else
    proto=zeros(n,m);
end
for ii=1:m
    if fast==2
        IND(offset+J,:) = [(ii-1)*n+J,(J-1)*m+ii];
        offset=offset+n;
    else
        for jj=1:n
            if fast
                K((ii-1)*n+jj,(jj-1)*m+ii)=1;
            else
                proto_ij=proto;
                proto_ij(jj,ii)=1;
                K((ii-1)*n+(1:n),(jj-1)*m+(1:m))=proto_ij;
            end
        end
    end
end
if fast==2
    siz=nm*ones(1,2);
    IND = sub2ind(siz,IND(:,1),IND(:,2));
    K(IND)=1;
end

end
