function [theta, theta_set] = gmm_estimation(D, data, num, options, dataset, directory)
theta_set = {};
%%EM parameters
log_text = '';
%% declare parameters
data = mxe_readdata(data, false);
%% initialize parameters
[theta, thetap] = initialization(data, num, options);
%% EM loop
llvec = zeros(1, options.totalItrNo);
[f0, f_ll0] = cost(D, data, theta, thetap, options);
itr = 1;
llvec(itr) = f0;
str = strcat(num2str(itr), ':', ', ', num2str(round(f_ll0,4)));
disp(str);
log_text = strcat(log_text , '\r\n' , str);
str = '----------------------------------------------------------------';
disp(str);
for itr = 2:options.totalItrNo
	component_weights = expectation(D, theta, data);
	theta = maximization(D, theta, thetap, data, component_weights, options);
	[f0, f_ll0] = cost(D, data, theta, thetap, options);
	llvec(itr) = f0;
	str = strcat(num2str(itr), ':', ', ', num2str(round(f_ll0,4)));
	disp(str);
	log_text = strcat(log_text , '\r\n' , str);
	str = '----------------------------------------------------------------';
	disp(str);
	theta_set{itr} = theta;
	if abs(llvec(itr)-llvec(itr-1)) < options.cost_threshold; break; end;
end
WriteToFile(dataset, log_text, directory)
end % of estimatelinesearch function
%-------------------------------------------------------------------------
function [f, f_ll] = cost(D, data, theta, thetap, options)
ndata = data.size();
f = -D.ll(theta, data)/ndata;
f_ll = f;
% if(options.use_prior == 1)
%     lp = -logprior(D, theta, thetap)/ndata;
%     f = f + lp;
% end
end
%-------------------------------------------------------------------------
function [theta, thetap] = initialization(data, num, options)
dim = data.dim;
ndata = data.size;
%% declare parameters
idx = 1:ndata;
idx = idx(randperm(ndata));
ndata_initial = ndata;
data_initial = data.data(:, idx);
data_initial = data_initial(:, 1:ndata_initial);
data_initial = mxe_readdata(data_initial, false);

theta = struct;
theta.p = ones(num, 1) ./ num;
Dmvn = mvnfactory(dim);
label = ceil(rand(1, ndata_initial) * num);
theta.D = cell(num, 1);
for k = 1:num
    theta0 = Dmvn.estimate(data_initial.data(:, label == k));
    theta.D{k}.mu = theta0.mu;
    theta.D{k}.sigma = theta0.sigma;
end

%% initialize prior parameters
thetap = struct;
if(options.use_prior == 1)
    thetap.mup = sum(data.data, 2)/ndata;
    thetap.kp = 0.01;
    thetap.nup = dim+2;
    thetap.sp = sum(diag(cov(data.data')))/dim/(num^(2/dim));
    thetap.Sp = cov(data.data')/(num^(2/dim));
    thetap.Spi = inv(thetap.Sp);
    thetap.ap = ones(num, 1)+1e-2;
end


end
%-------------------------------------------------------------------------
function component_weights = expectation(D, theta, data)
component_weights = D.weighting(theta, data);
end
%-------------------------------------------------------------------------
function theta = maximization(D, theta, thetap, data, component_weights, options)
num = length(theta.D);
dim = data.dim;
ndata = data.size;

sumW = sum(component_weights, 2);
sumWInv = 1./sumW;
mu0 = component_weights*data.data';
%% update mu
mu = mu0;
if(options.use_prior==1)
    mu = bsxfun(@plus, mu, thetap.kp*thetap.mup');
    mu = bsxfun(@times, mu , 1./(thetap.kp+sumW));
else
    mu = bsxfun(@times, mu , sumWInv);
end
for k = 1:num
    theta.D{k}.mu = mu(k,:)';
end
%% update sigma
mu = mu0;
mu = bsxfun(@times, mu , sumWInv);
for k = 1:num
    wightsk = sqrt(component_weights(k,:));
    X = data.data;
    muk = mu(k,:)';
    Xo = bsxfun(@minus, X, muk);
    Xo = bsxfun(@times, Xo, wightsk);
    Wk = Xo*Xo';
    if(options.use_prior==1)
        sigmak =  (muk-thetap.mup) * (muk-thetap.mup)';
        sigmak = thetap.kp * sumW(k) / (thetap.kp+sumW(k)) * sigmak;
        sigmak = sigmak + thetap.Sp;
        sigmak = sigmak + Wk;
        sigmak = sigmak / (thetap.nup + sumW(k) + dim + 2);
    else
        sigmak =  Wk/sumW(k);
    end
    theta.D{k}.sigma = sigmak;
end
%% update p
if (options.use_prior==1)
    theta.p = (thetap.ap-1+sumW)/(ndata+sum(thetap.ap-1));
else
    theta.p = sumW/ndata;
end

end
%-------------------------------------------------------------------------
function WriteToFile(filename, text,foldername)
disp(text)
filename = string(filename);
% foldername = string(foldername);
mkdir_if_not_exist(foldername)
filename = strcat(foldername,'/', filename,'.txt');
%open file with write permission
fid = fopen(filename, 'wt');
%write a line of text
fprintf(fid, '%s\n', text);
%close filefprintf newlinefprmatla
fclose(fid);
end
%-------------------------------------------------------------------------
